#!/usr/bin/python

# would you like some nice Fish.py?

import re
import textwrap
import gtk
import gtk.gdk
import gtk.glade
import gnome
import feedparser
import striptags
from threading import Thread
import random
from blogfish.blogfish_functions import debug, get_config_value

Blog_cache = {}
gtk.timeout_add(300, lambda *junk: Blog_cache.clear() or gtk.TRUE)

def flip_pixbuf(pb):
	if not pb.get_has_alpha():
		pb = pb.add_alpha(0, "0", "0", "0")
	bps = pb.get_bits_per_sample()
	# what if there are not 4 channels
	rs = pb.get_rowstride()
	pix = pb.get_pixels()
		
	newpix = ""
	x = rs
	pixstride = pb.get_n_channels() * bps/8
	while x <= len(pix):
		row = pix[x - rs : x]
		newrow = ""
		x = x + rs
		y = pixstride
		while y <= len(row):
			sample = row[y - pixstride : y]
			y = y + pixstride
			newrow = sample + newrow
		newpix = newpix + newrow
		
	return gtk.gdk.pixbuf_new_from_data(newpix, pb.get_colorspace(), \
			gtk.TRUE, bps, pb.get_width(), \
			pb.get_height(), rs)

class Fish:
	def __init__(self, name, pixbuf, email=None, URL=None, blogURL=None, \
			words = None, fitness=5, sex=None, age=20, \
			facingleft=0, x=-15, y=0):
		for k in 'email', 'name', 'URL', 'blogURL', 'words', \
				'facingleft', 'x', 'y', 'pixbuf', \
				'fitness', 'age':
			self.__dict__[k] = vars()[k]
		
		self.reversed_pixbuf = 0
		self.climb = 0
		self.already_voted_on = 0
		if not sex: sex = random.choice(("M","F"))
		self.sex = sex
		self.blog_thread = None
		self.fetch_blog_entry() 
		gtk.timeout_add(900, self.fetch_blog_entry)
		
	def get_gdk_pixbuf(self):
		if self.blog_thread and self.blog_thread.isAlive():
			self.blog_thread.join(0.05) # a small delay
		if self.facingleft:
			return self.pixbuf
		elif self.reversed_pixbuf:
			return self.reversed_pixbuf

		self.reversed_pixbuf = flip_pixbuf(self.pixbuf)
		return self.reversed_pixbuf
	
	def fetch_blog_entry(self):
		global Blog_cache
		self.blog_entry = None
		if self.blog_thread: return
		if not self.blogURL: 
			self.blog_entry = None
			return
		if Blog_cache.has_key(self.blogURL):
			self.blog_entry = Blog_cache[self.blogURL]
			return
		self.blog_thread = Thread(None, self._do_fetch_blog_entry)
		
		self.blog_thread.setDaemon(1)
		self.blog_thread.start()
		self.blog_thread.join(2.0) # give it a chance!

	def _do_fetch_blog_entry(self):
		global Blog_cache
		d = feedparser.parse(self.blogURL)
		if len(d.entries):
		    d.entries.sort(self._sort_blog_entries)
		    Blog_cache[self.blogURL] = d.entries[0]
		    debug("setting blog entry")
		    self.blog_entry = d.entries[0]
		else:
		    self.blog_entry = None

	def _sort_blog_entries(self, a, b):
		return cmp(b.modified_parsed, a.modified_parsed)
	
	def get_text(self):
		b = self.blog_entry
		if b and b.has_key("title"): return b.title
		if self.words: return self.words

	def get_long_text(self):
		"""returns blog entry as pango markup"""
		b = self.blog_entry
		if not b: return ""
		if b.has_key("summary"):
			sum = b.summary
			if b.has_key("summary_detail"): type = b.summary_detail.type
			else: type = "text/html"
			if type == "text/html" or type == "application/xhtml+xml":
				# remove html
				sum = striptags.strip(sum)
				sum = re.sub("\n+", "\n", sum)
				sum = sum.strip()
			if b.has_key("title"):
				sum = "<b>%s</b>\n\n" % b.title + sum
			return sum
		elif  b.has_key("title"):
			return b.title
		# give up
		return ""
	
	def to_string(self):
		if self.words: words = self.words.replace("\n", " ")
		else: words = ""
		fstr = "name:%s\nsex:%s\nemail:%s\nURL:%s\nblogURL:%s\n" % \
				(self.name, self.sex, self.email, self.URL, \
				self.blogURL)
		fstr = fstr + "words:%s\nfitness:%s\nrowstride:%s" % \
				(words, self.fitness, \
				self.pixbuf.get_rowstride())
		fstr = fstr + "\n\n" + self.pixbuf.get_pixels()
		return fstr

	def show_properties_dialog(self, applet):
		glade_dir = get_config_value("gladeDir")
		xml = gtk.glade.XML(glade_dir + "/fish_dialog.glade")
		w = {}
		for widget in 'fish_image', 'table', 'plus_button', 'minus_button', \
				'sex_label', 'name_label', 'fitness_label', \
				'main_eventbox', 'vbox', 'window': 
			w[widget] = xml.get_widget(widget)
			if not w[widget]: print "couldn't get %s" % widget
		
		w['name_label'].set_markup("<b>" + self.name + "</b>")
		sexes={'M':'Male', 'F':'Female'}
		w['sex_label'].set_markup(sexes[self.sex])
		
		w['fitness_label'].set_markup("%s" % self.fitness)
		w['fish_image'].set_from_pixbuf(self.get_gdk_pixbuf())

		table = w['table']
		pw = w['window']
		
		def add_row(left, right):
			rows = table.get_property('n-rows')
			table.resize(rows + 1, 2)
			left.set_use_markup(1)
			left.set_alignment(0, 0.5)
			table.attach(left, 0, 1, rows, rows + 1)
			table.attach(right, 1, 2, rows, rows + 1)
		if self.words:
			wl = gtk.Label(self.words)
			wl.set_alignment(0, 0.5)
			add_row(gtk.Label("<b>Message</b>"), wl)
		if self.email:
			el = gtk.Label(self.email)
			el.set_alignment(0, 0.5)
			add_row(gtk.Label("<b>Email</b>"), el)
		if self.URL:
			url_text = self.URL
			if len(url_text) > 20: url_text=url_text[0:16] + "..."
			URLLabel = gtk.Label("<span foreground=\"blue\"" \
					+ "underline=\"single\">" \
					+ url_text + "</span>")
			URLLabel.set_use_markup(gtk.TRUE)
			URLLabel.set_justify(gtk.JUSTIFY_LEFT)
			URLLabel.set_alignment(0, 0.5)
			ebURL = gtk.EventBox()
			ebURL.add(URLLabel)
			tt = gtk.Tooltips()
			tt.set_tip(ebURL, self.URL)
			ebURL.connect("button_press_event", self.URL_button_press, \
					self.URL, pw)
			add_row(gtk.Label("<b>URL</b>"), ebURL)

		text = self.get_long_text()
		if text: 
			debug("adding row for blog entry")
			text = textwrap.fill(text, 50, replace_whitespace=0)
			text = "\n".join(text.splitlines()[0:15])
			rows = table.get_property('n-rows')
			table.resize(rows + 1, 2)
			blogLabel = gtk.Label('<span size="x-small">' \
					+ text + '</span>')
			blogLabel.set_use_markup(gtk.TRUE)
			blogLabel.set_justify(gtk.JUSTIFY_LEFT)
			ebBlog = gtk.EventBox()
			ebBlog.add(blogLabel)
			ebBlog.set_border_width(2)
			table.attach(ebBlog, 0, 2, rows, rows + 1)

		w['plus_button'].connect("clicked", self.adjust_fitness, +1, pw)
		w['minus_button'].connect("clicked", self.adjust_fitness, -1, pw)
		w['main_eventbox'].connect("button_press_event", lambda win, ev: pw.destroy()) 
				
		if self.already_voted_on: 
		    w['plus_button'].set_sensitive(0)
		    w['minus_button'].set_sensitive(0)
		
		bgcolor = gtk.gdk.color_parse("#AABBFF")
		table.foreach(lambda wid: wid.modify_bg(gtk.STATE_NORMAL, bgcolor))
		for but in w['plus_button'], w['minus_button']:
			for state in gtk.STATE_NORMAL, gtk.STATE_ACTIVE, \
					gtk.STATE_PRELIGHT, gtk.STATE_INSENSITIVE:
				but.modify_bg(state, bgcolor)
		pw.modify_bg(gtk.STATE_NORMAL, bgcolor)
		
		pw.get_child().modify_bg(gtk.STATE_NORMAL, bgcolor)
		pw.show_all()	
		# can only do this here, alas!
		if vars().has_key("ebURL"): 
			ebURL.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
		if vars().has_key("ebBlog") and self.blog_entry \
				and self.blog_entry.has_key("link"): 
			ebBlog.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
			tt2 = gtk.Tooltips()
			tt2.set_tip(ebBlog, self.blog_entry.link)
			ebBlog.connect("button_press_event", self.URL_button_press, \
					self.blog_entry.link, pw)
		
	def URL_button_press(self, eb, event, link, pw):
		if event.button == 1:
			gnome.url_show(link)
		return 1
		
	def adjust_fitness(self, button, amount, pw):
		pw.destroy()
		if self.already_voted_on: return
		self.already_voted_on = 1
		self.fitness = self.fitness + amount
		return 1
		

def fish_from_string(string):
	(attrs, pb_data) = string.split("\n\n", 1)
	name = "Anonymous Fish"
	email = URL = blogURL = None
	fitness = 1
	for line in attrs.splitlines():
		if ":" not in line: continue
		k,v = line.split(":",1)
		if k == "email": email = v
		if k == "sex": sex = v
		if k == "name":  name = v
		if k == "URL":   URL = v
		if k == "words": words = v
		if k == "blogURL": blogURL = v
		if k == "rowstride": rowstride = int(v)
		if k == "fitness": fitness = int(v)
	pixbuf = gtk.gdk.pixbuf_new_from_data(pb_data, gtk.gdk.COLORSPACE_RGB, \
			1, 8, 15, 15, rowstride)
	return Fish(name, pixbuf, email, URL, blogURL, words, fitness, sex)
